package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DEROutputStream;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.x509.X509ObjectIdentifiers;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.digests.RIPEMD160Digest;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.digests.SHA224Digest;
import cn.com.infosec.crypto.digests.SHA256Digest;
import cn.com.infosec.crypto.digests.SHA384Digest;
import cn.com.infosec.crypto.digests.SHA512Digest;
import cn.com.infosec.crypto.params.AsymmetricKeyParameter;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.signers.DSASigner;
import cn.com.infosec.crypto.signers.ECDSASigner;
import cn.com.infosec.crypto.signers.ECNRSigner;
import cn.com.infosec.jce.interfaces.ECKey;
import cn.com.infosec.jce.interfaces.ECPublicKey;
import cn.com.infosec.jce.interfaces.GOST3410Key;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.SignatureSpi;
import java.security.interfaces.DSAKey;
import java.security.spec.AlgorithmParameterSpec;

public class JDKDSASigner extends SignatureSpi implements PKCSObjectIdentifiers, X509ObjectIdentifiers {
   private Digest digest;
   private DSA signer;
   private SecureRandom random;

   protected JDKDSASigner(Digest digest, DSA signer) {
      this.digest = digest;
      this.signer = signer;
   }

   private byte[] derEncode(BigInteger r, BigInteger s) {
      ByteArrayOutputStream var3;
      ByteArrayOutputStream var10000 = var3 = new ByteArrayOutputStream;
      var10000.<init>();
      DEROutputStream var10001 = new DEROutputStream(var3);
      ASN1EncodableVector var4;
      ASN1EncodableVector var10002 = var4 = new ASN1EncodableVector;
      var10002.<init>();
      var10002.add(new DERInteger(r));
      var10002.add(new DERInteger(s));
      var10001.writeObject(new DERSequence(var4));
      return var10000.toByteArray();
   }

   private BigInteger[] derDecode(byte[] encoding) {
      ASN1Sequence var2 = (ASN1Sequence)(new ASN1InputStream(encoding)).readObject();
      return new BigInteger[]{((DERInteger)var2.getObjectAt(0)).getValue(), ((DERInteger)var2.getObjectAt(1)).getValue()};
   }

   protected void engineInitVerify(PublicKey publicKey) {
      AsymmetricKeyParameter publicKey1;
      if (publicKey instanceof ECPublicKey) {
         publicKey1 = ECUtil.generatePublicKeyParameter(publicKey);
      } else if (publicKey instanceof GOST3410Key) {
         publicKey1 = GOST3410Util.generatePublicKeyParameter(publicKey);
      } else if (publicKey instanceof DSAKey) {
         publicKey1 = DSAUtil.generatePublicKeyParameter(publicKey);
      } else {
         PublicKey var10000;
         boolean var10001;
         try {
            var10000 = JDKKeyFactory.createPublicKeyFromDERStream(publicKey.getEncoded());
         } catch (Exception var7) {
            var10001 = false;
            throw new InvalidKeyException("can't recognise key type in DSA based signer");
         }

         publicKey = var10000;

         boolean var9;
         try {
            var9 = var10000 instanceof ECPublicKey;
         } catch (Exception var6) {
            var10001 = false;
            throw new InvalidKeyException("can't recognise key type in DSA based signer");
         }

         AsymmetricKeyParameter var10;
         if (var9) {
            try {
               var10 = ECUtil.generatePublicKeyParameter(publicKey);
            } catch (Exception var4) {
               var10001 = false;
               throw new InvalidKeyException("can't recognise key type in DSA based signer");
            }
         } else {
            try {
               var9 = publicKey instanceof DSAKey;
            } catch (Exception var5) {
               var10001 = false;
               throw new InvalidKeyException("can't recognise key type in DSA based signer");
            }

            if (!var9) {
               try {
                  throw new InvalidKeyException("can't recognise key type in DSA based signer");
               } catch (Exception var2) {
                  var10001 = false;
                  throw new InvalidKeyException("can't recognise key type in DSA based signer");
               }
            }

            try {
               var10 = DSAUtil.generatePublicKeyParameter(publicKey);
            } catch (Exception var3) {
               var10001 = false;
               throw new InvalidKeyException("can't recognise key type in DSA based signer");
            }
         }

         publicKey1 = var10;
      }

      this.digest.reset();
      this.signer.init(false, publicKey1);
   }

   protected void engineInitSign(PrivateKey privateKey, SecureRandom random) {
      this.random = random;
      this.engineInitSign(privateKey);
   }

   protected void engineInitSign(PrivateKey privateKey) {
      AsymmetricKeyParameter privateKey1;
      if (privateKey instanceof ECKey) {
         privateKey1 = ECUtil.generatePrivateKeyParameter(privateKey);
      } else if (privateKey instanceof GOST3410Key) {
         privateKey1 = GOST3410Util.generatePrivateKeyParameter(privateKey);
      } else {
         privateKey1 = DSAUtil.generatePrivateKeyParameter(privateKey);
      }

      this.digest.reset();
      SecureRandom var2;
      if ((var2 = this.random) != null) {
         DSA var10000 = this.signer;
         ParametersWithRandom var3;
         var3 = new ParametersWithRandom.<init>(privateKey1, var2);
         var10000.init(true, var3);
      } else {
         this.signer.init(true, privateKey1);
      }

   }

   protected void engineUpdate(byte b) {
      this.digest.update(b);
   }

   protected void engineUpdate(byte[] b, int off, int len) {
      this.digest.update(b, off, len);
   }

   protected byte[] engineSign() {
      JDKDSASigner var10000 = this;
      JDKDSASigner var10001 = this;
      JDKDSASigner var10002 = this;
      byte[] var3 = new byte[this.digest.getDigestSize()];
      var10002.digest.doFinal(var3, 0);

      Exception var6;
      label25: {
         boolean var7;
         BigInteger[] var8;
         BigInteger var9;
         try {
            var8 = var10001.signer.generateSignature(var3);
            var9 = var8[0];
         } catch (Exception var2) {
            var6 = var2;
            var7 = false;
            break label25;
         }

         BigInteger var4 = var9;

         try {
            return var10000.derEncode(var4, var8[1]);
         } catch (Exception var1) {
            var6 = var1;
            var7 = false;
         }
      }

      Exception var5 = var6;
      throw new SignatureException(var5.toString());
   }

   protected boolean engineVerify(byte[] sigBytes) {
      JDKDSASigner var10000 = this;
      JDKDSASigner var10001 = this;
      byte[] var10002 = sigBytes;
      JDKDSASigner var10003 = this;
      byte[] var3 = new byte[this.digest.getDigestSize()];
      var10003.digest.doFinal(var3, 0);

      BigInteger[] var7;
      try {
         var7 = var10001.derDecode(var10002);
      } catch (Exception var2) {
         throw new SignatureException("error decoding signature bytes.");
      }

      BigInteger[] sigBytes1 = var7;
      DSA var6 = var10000.signer;
      byte[] var8 = var3;
      BigInteger var4 = sigBytes1[0];
      return var6.verifySignature(var8, var4, sigBytes1[1]);
   }

   protected void engineSetParameter(AlgorithmParameterSpec params) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   /** @deprecated */
   protected void engineSetParameter(String param, Object value) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   /** @deprecated */
   protected Object engineGetParameter(String param) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   private static class NullDigest implements Digest {
      private ByteArrayOutputStream bOut;

      private NullDigest() {
         NullDigest var10000 = this;
         super();
         ByteArrayOutputStream var1;
         var1 = new ByteArrayOutputStream.<init>();
         var10000.bOut = var1;
      }

      // $FF: synthetic method
      NullDigest(NullDigest var1) {
         this();
      }

      public String getAlgorithmName() {
         return "NULL";
      }

      public int getDigestSize() {
         return this.bOut.size();
      }

      public void update(byte in) {
         this.bOut.write(in);
      }

      public void update(byte[] in, int inOff, int len) {
         this.bOut.write(in, inOff, len);
      }

      public int doFinal(byte[] out, int outOff) {
         byte[] var10000 = this.bOut.toByteArray();
         int var3 = var10000.length;
         System.arraycopy(var10000, 0, out, outOff, var3);
         return var10000.length;
      }

      public void reset() {
         this.bOut.reset();
      }
   }

   public static class dsa224 extends JDKDSASigner {
      public dsa224() {
         dsa224 var10000 = this;
         SHA224Digest var2;
         var2 = new SHA224Digest.<init>();
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class dsa256 extends JDKDSASigner {
      public dsa256() {
         dsa256 var10000 = this;
         SHA256Digest var2;
         var2 = new SHA256Digest.<init>();
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class dsa384 extends JDKDSASigner {
      public dsa384() {
         dsa384 var10000 = this;
         SHA384Digest var2;
         var2 = new SHA384Digest.<init>();
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class dsa512 extends JDKDSASigner {
      public dsa512() {
         dsa512 var10000 = this;
         SHA512Digest var2;
         var2 = new SHA512Digest.<init>();
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSA extends JDKDSASigner {
      public ecDSA() {
         ecDSA var10000 = this;
         SHA1Digest var2;
         var2 = new SHA1Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSA224 extends JDKDSASigner {
      public ecDSA224() {
         ecDSA224 var10000 = this;
         SHA224Digest var2;
         var2 = new SHA224Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSA256 extends JDKDSASigner {
      public ecDSA256() {
         ecDSA256 var10000 = this;
         SHA256Digest var2;
         var2 = new SHA256Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSA384 extends JDKDSASigner {
      public ecDSA384() {
         ecDSA384 var10000 = this;
         SHA384Digest var2;
         var2 = new SHA384Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSA512 extends JDKDSASigner {
      public ecDSA512() {
         ecDSA512 var10000 = this;
         SHA512Digest var2;
         var2 = new SHA512Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecDSARipeMD160 extends JDKDSASigner {
      public ecDSARipeMD160() {
         ecDSARipeMD160 var10000 = this;
         RIPEMD160Digest var2;
         var2 = new RIPEMD160Digest.<init>();
         ECDSASigner var1;
         var1 = new ECDSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecNR extends JDKDSASigner {
      public ecNR() {
         ecNR var10000 = this;
         SHA1Digest var2;
         var2 = new SHA1Digest.<init>();
         ECNRSigner var1;
         var1 = new ECNRSigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecNR224 extends JDKDSASigner {
      public ecNR224() {
         ecNR224 var10000 = this;
         SHA224Digest var2;
         var2 = new SHA224Digest.<init>();
         ECNRSigner var1;
         var1 = new ECNRSigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecNR256 extends JDKDSASigner {
      public ecNR256() {
         ecNR256 var10000 = this;
         SHA256Digest var2;
         var2 = new SHA256Digest.<init>();
         ECNRSigner var1;
         var1 = new ECNRSigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecNR384 extends JDKDSASigner {
      public ecNR384() {
         ecNR384 var10000 = this;
         SHA384Digest var2;
         var2 = new SHA384Digest.<init>();
         ECNRSigner var1;
         var1 = new ECNRSigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class ecNR512 extends JDKDSASigner {
      public ecNR512() {
         ecNR512 var10000 = this;
         SHA512Digest var2;
         var2 = new SHA512Digest.<init>();
         ECNRSigner var1;
         var1 = new ECNRSigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class noneDSA extends JDKDSASigner {
      public noneDSA() {
         noneDSA var10000 = this;
         NullDigest var2;
         var2 = new NullDigest.<init>((NullDigest)null);
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class stdDSA extends JDKDSASigner {
      public stdDSA() {
         stdDSA var10000 = this;
         SHA1Digest var2;
         var2 = new SHA1Digest.<init>();
         DSASigner var1;
         var1 = new DSASigner.<init>();
         var10000.<init>(var2, var1);
      }
   }
}
